package com.lodgon.dali.core.oauth.ejb;

import com.lodgon.dali.core.oauth.entity.Attribute;
import com.lodgon.dali.core.oauth.entity.DaliServiceConsumer;
import com.lodgon.dali.core.oauth.entity.DaliToken;
import com.lodgon.dali.core.oauth.entity.OAuthUser;
import com.lodgon.dali.core.oauth.util.StringUtil;
import java.security.Principal;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

/**
 *
 * @author joeri
 */
@Stateless
public class OAuthBean {
  @PersistenceContext
  EntityManager em;

  public DaliToken createRequestToken(String consumerKey, String callback, Map<String, List<String>> attributes) {
    // generate Attribute list
    List<Attribute> attrs = new LinkedList<Attribute>();
    for (Map.Entry<String, List<String>> entry : attributes.entrySet()) {
      for (String value : entry.getValue()) {
        attrs.add(new Attribute(entry.getKey(), value));
      }
    }

    DaliServiceConsumer daliServiceConsumer = findDaliServiceConsumerByConsumerKey(consumerKey);

    // create request token
    DaliToken requestToken = new DaliToken();
    requestToken.setType(DaliToken.Type.OAUTH_REQUEST_TOKEN);
    requestToken.setToken(StringUtil.getSecureRandomString(32));
    requestToken.setSecret(StringUtil.getSecureRandomString(32));
    requestToken.setDaliServiceConsumer(daliServiceConsumer);
    requestToken.setAttrs(attrs);
    requestToken.setCallback(callback);
		requestToken.setCreationDate(System.currentTimeMillis());

    em.persist(requestToken);
    return requestToken;
  }

  public DaliToken createAccessToken(String token, String verifier) {
    DaliToken requestToken = findDaliToken(token, DaliToken.Type.OAUTH_REQUEST_TOKEN);
    if (requestToken != null) {
      if (verifier == null || ! verifier.equals(requestToken.getVerifier())) {
        return null;
      }

      Principal principal = requestToken.getPrincipal();
      if (principal == null) {
        return null;
      }

      DaliServiceConsumer daliServiceConsumer = requestToken.getDaliServiceConsumer();
      OAuthUser oauthUser = requestToken.getOauthUser();

      // see if there is a valid access token
			DaliToken validAccessToken = null;
      List<DaliToken> previousAccessTokens = findDaliTokensByConsumerAndPrincipal(daliServiceConsumer.getId(), oauthUser.getId(), DaliToken.Type.OAUTH_ACCESS_TOKEN);
      for (DaliToken previousAccessToken : previousAccessTokens) {				
        validAccessToken = previousAccessToken;
				break;
      }

      // copy attributes and add userId attribute
      List<Attribute> attrs = new LinkedList<Attribute>();
      attrs.addAll(requestToken.getAttrs());
      attrs.add(new Attribute("userId", oauthUser.getUid()));

      // remove request token
      em.remove(requestToken);

			if (validAccessToken == null) {
				// create new access token
				DaliToken accessToken = new DaliToken();
				accessToken.setType(DaliToken.Type.OAUTH_ACCESS_TOKEN);
				accessToken.setToken(StringUtil.getSecureRandomString(32)); 
				accessToken.setSecret(StringUtil.getSecureRandomString(32)); 
				accessToken.setDaliServiceConsumer(daliServiceConsumer);
				accessToken.setOauthUser(oauthUser);
				accessToken.setAttrs(attrs);
				accessToken.setCreationDate(System.currentTimeMillis());

				em.persist(accessToken);
				return accessToken;
			} else {
				em.detach(validAccessToken);
				validAccessToken.setAttrs(attrs);
				return validAccessToken;
			}
    } else {
      return null;
    }
  }
  
  public DaliToken findDaliToken(String token, DaliToken.Type type) {
    Query q = em.createNamedQuery("DaliToken.findByTokenAndType");
    q.setParameter("token", token);
    q.setParameter("type", type);
    List<DaliToken> tokens = q.getResultList();
    if (tokens.isEmpty()) {
      return null;
    } else {
      return tokens.iterator().next();
    }
  }

  public List<DaliToken> findDaliTokensByConsumerAndPrincipal(int consumerId, int principalId, DaliToken.Type type) {
    Query q = em.createNamedQuery("DaliToken.findByConsumerAndPrincipal");
    q.setParameter("consumer", em.find(DaliServiceConsumer.class, consumerId));
    q.setParameter("principal", em.find(OAuthUser.class, principalId));
    q.setParameter("type", type);
    return q.getResultList();
  }
  
  public String authorizeServiceConsumer(int userId, String token, boolean authorized) {
    DaliToken requestToken = findDaliToken(token, DaliToken.Type.OAUTH_REQUEST_TOKEN);
    if (requestToken != null) {
      if (authorized) {
        OAuthUser user = em.find(OAuthUser.class, userId);
        if (user != null) {
          requestToken.setOauthUser(em.find(OAuthUser.class, userId));
          requestToken.setVerifier(StringUtil.getSecureRandomString(16));
          em.merge(requestToken);
          return requestToken.getVerifier();
        } else {
          return null;
        }
      } else {
        em.remove(requestToken);
        return null;
      }
    } else {
      return null;
    }
  }

  public List<DaliToken> findAuthorizedConsumers(int userId) {
    Query q = em.createNamedQuery("DaliToken.findByPrincipal");
    q.setParameter("principal", em.find(OAuthUser.class, userId));
    q.setParameter("type", DaliToken.Type.OAUTH_ACCESS_TOKEN);
    return q.getResultList();
  }
  
  public void revokeAuthorization(int consumerId, int userId) {
    List<DaliToken> accessTokens = findDaliTokensByConsumerAndPrincipal(consumerId, userId, DaliToken.Type.OAUTH_ACCESS_TOKEN);
    for (DaliToken accessToken : accessTokens) {
      em.remove(accessToken);
    }
  }
	
	
	public void removeUnusedDaliTokensOlderThan(DaliToken.Type type, long date) {
    Query q = em.createNamedQuery("DaliToken.findUnusedByTypeOlderThan");
    q.setParameter("type", type);
    q.setParameter("date", date);
    List<DaliToken> list = q.getResultList();		
		for(DaliToken dt : list) {
			em.remove(dt);
		}
	}
	

  public DaliServiceConsumer createDaliServiceConsumer(DaliServiceConsumer daliServiceConsumer) {
    em.persist(daliServiceConsumer);
    return daliServiceConsumer;
  }

  public DaliServiceConsumer updateDaliServiceConsumer(DaliServiceConsumer daliServiceConsumer) {
    return em.merge(daliServiceConsumer);
  }

  public void deleteDaliServiceConsumer(int daliServiceConsumerId) {
    DaliServiceConsumer daliServiceConsumer = em.find(DaliServiceConsumer.class, daliServiceConsumerId);
    if (daliServiceConsumer != null) {
      em.remove(daliServiceConsumer);
    }
  }

  public DaliServiceConsumer findDaliServiceConsumer(int daliServiceConsumerId) {
    return em.find(DaliServiceConsumer.class, daliServiceConsumerId);
  }

  public List<DaliServiceConsumer> findAllDaliServiceConsumers() {
    return findAllDaliServiceConsumers(0, -1);
  }

  public List<DaliServiceConsumer> findAllDaliServiceConsumers(int offset, int max) {
    Query q = em.createNamedQuery("DaliServiceConsumer.findAll");
    q.setFirstResult(offset);
    if (max > 0) {
      q.setMaxResults(max);
    }
    return q.getResultList();
  }
  
  public Long countDaliServiceConsumers() {
    Query q = em.createNamedQuery("DaliServiceConsumer.countAll");
    return (Long) q.getSingleResult();
  }

  public DaliServiceConsumer findDaliServiceConsumerByConsumerKey(String consumerKey) {
    Query q = em.createNamedQuery("DaliServiceConsumer.findByConsumerKey");
    q.setParameter("consumerKey", consumerKey);
    List<DaliServiceConsumer> daliServiceConsumers = q.getResultList();
    if (daliServiceConsumers.isEmpty()) {
      return null;
    } else {
      return daliServiceConsumers.iterator().next();
    }
  }
}
